#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ________________ HEADER _________________________


"""parse_skt module.

Program to parse an input CDF skeleton table file.

"""


# ________________ IMPORT _________________________
# (Include here the modules to import, e.g. import sys)
import logging

from maser.tools.cdf.serializer.exceptions import InvalidEntry
from maser.tools.cdf.serializer.txt import Skt2txt, Txt2skt
from maser.tools.cdf.serializer.xlsx import Skt2xlsx, Xlsx2skt
from maser.tools.cdf.serializer.globals import SHEETS, HEADER, GATTRS, ZVARS, VATTRS

__all__ = ["Skeleton"]

# ________________ Global Variables _____________
# (define here the global variables)
logger = logging.getLogger(__name__)


# ________________ Class Definition __________
# (If required, define here classes)
class Skeleton:
    def __init__(self):
        """
        Init method of the Skeleton class.

        :param skeleton_file:
        """

        self.file = None
        self.cdf_items = dict()
        self.header = dict()
        self.gattrs = dict()
        self.vattrs = dict()
        self.vattrList = []
        self.zvars = dict()
        self.xlsx = False

    @staticmethod
    def from_txt(txt_file):
        """
        Create a Skeleton instance from the input text file.

        :param txt_file:
        :return: an instance of Skeleton
        """

        skeleton = Skeleton()
        return Txt2skt(skeleton).parse_txt(txt_file)

    @staticmethod
    def from_xlsx(xlsx_file, auto_pad=True):
        """
        Create a Skeleton instance from the input Excel file.

        :param xlsx_file: input Excel file to convert to Skeleton object
        :param auto_pad: If True assigns automatic value to VAR_PADVALUE
        :return: an instance of Skeleton
        """
        skeleton = Skeleton()
        return Xlsx2skt(skeleton).parse_xlsx(xlsx_file, auto_pad=auto_pad)

    def to_txt(self, output_path=None, overwrite=False):
        """
        Convert a Skeleton object into an output Skeleton table file in ASCII format.

        :param output_path: Path of the output Skeleton table file
        :param overwrite: If True, the overwrite existing output file
        :return: Path of the output file
        """

        return Skt2txt(self).write_txt(output_path=output_path, overwrite=overwrite)

    def to_xlsx(self, output_path=None, overwrite=False):
        """
        Convert a Skeleton object into an output Excel 2007 format file.

        :param output_path: Path of the output Excel file
        :param overwrite: If True, the overwrite existing output file
        :return: Path of the output file
        """

        return Skt2xlsx(self).write_xlsx(output_path=output_path, overwrite=overwrite)

    def is_valid_header(self, header, ignore=[]):
        """
        Check that the input is a valid cdf header.

        :param header: header dictionary
        :param ignore: List of field(s) to ignore
        :return: True if the header is valid
        """

        for entry in header:
            for field in SHEETS[HEADER]:
                if field in ignore:
                    continue
                if field not in entry:
                    logger.warning("{0} field is missing!".format(field))
                    return False

        return True

    def is_valid_gatt(self, entries, ignore=[]):
        """
        Check that the input gattr entries are valid.

        :param entries: Entries to check
        :param ignore: list of field(s) to ignore
        :return: True if it is valid
        """
        for entry in entries:
            for field in SHEETS[GATTRS]:
                if field in ignore:
                    continue
                if field not in entry:
                    logger.warning("{0} field is missing!".format(field))
                    return False

        return True

    def is_valid_zvar(self, entry, ignore=[]):
        """
        Check that the input is a valid zvars entry.

        :param entry: entry to check
        :param ignore: list of field(s) to ignore
        :return: True if it is valid
        """
        for field in SHEETS[ZVARS]:
            if field in ignore:
                continue
            if field not in entry:
                logger.warning("{0} field is missing!".format(field))
                return False

        return True

    def is_valid_vattrs(self, entry, ignore=[]):
        """
        Check that the input is a valid vattrs.

        :param entry: entry to check
        :param ignore: list of field(s) to ignore
        :return: True if it is valid
        """

        for field in SHEETS[VATTRS]:
            if field in ignore:
                continue
            if field not in entry:
                logger.warning("{0} field is missing!".format(field))
                return False

        return True

    def set_header(self, entry):
        """
        Set header field(s)

        :param entry: dictionary containing header field(s) to udpate
        :return:
        """
        for key, val in entry.items():
            if key not in SHEETS[HEADER]:
                logger.warning("Unknown field: {0}!".format(key))
                continue
            self.header[key] = val

        return True

    def add_gattr(self, attname, entries, append=False):
        """
        Add a global attribute to the Skeleton object.


        :param attname: name of the g. attribute to add
        :param entries: List containing the entries for the g.attribute
        :param append: If True, then append input entries to the existing one(s)
        """

        logger.info("Adding entries for g.attribute {0}".format(attname))
        if attname in self.gattrs and append:
            # Check that all the expected fields are in the input entries
            if not self.is_valid_gatt(
                entries, ignore=["Entry Number", "Attribute Name", "Data Type"]
            ):
                raise InvalidEntry

            # Get number of entries
            nentry = len(self.gattrs[attname])
            for i, entry in enumerate(entries):
                entry["Attribute Name"] = attname
                entry["Data Type"] = self.gattrs[attname][0]["Data Type"]
                entry["Entry Number"] = str(nentry + 1)
                self.gattrs[attname].append(entry)

            return True

        elif attname in self.gattrs:
            logger.warning("{0} already exists!".format(attname))
            return False
        else:
            # Check that all the expected fields are in the input entries
            if not self.is_valid_gatt(entries):
                raise InvalidEntry

            self.gattrs[attname] = entries
            self.header["ngattr"] = str(int(self.header["ngattr"]) + 1)
            return True

    def set_gattr(self, attname, new_entry, add=False):
        """
        Update an entry for a given g.attribute in the Skeleton object.

        :param attname: name of the g.attribute to update
        :param new_entry: new entry values provided as a dictionary
        :param add: If g.attribute does not exist, then add it with the input entries
        :return:
        """
        logger.info("Updating g.attribute {0}".format(attname))
        # Check if the attribute already exists
        if attname not in self.gattrs:
            logger.warning("{0} g.attribute does not exist!".format(attname))
            if not add:
                return False
            else:
                return self.add_gattr(attname, [new_entry])

        # Check if the Entry Number is provided in the new_entry dictionary
        if "Entry Number" not in new_entry:
            logger.error("No Entry Number provided!")
            raise InvalidEntry

        # Loop over self.gattrs[attname] to look for the entry number to update
        has_entry = False
        for i, entry in enumerate(self.gattrs[attname]):
            # If found, then retrieve the new entry values and save them into self.gattrs[attname]
            if self.gattrs[attname][i]["Entry Number"] == new_entry["Entry Number"]:
                for key, val in new_entry.items():
                    if key not in SHEETS[GATTRS]:
                        logger.warning("Unknown field: {0}!".format(key))
                        continue
                    self.gattrs[attname][i][key] = new_entry[key]
                    has_entry = True

        if has_entry is False:
            logger.warning(
                "Entry Number {0} not found!".format(new_entry["Entry Number"])
            )

        return has_entry

    def rename_gattr(self, old_attname, new_attname):
        """
        Rename a global attr. from the Skeleton object.

        :param old_attname:
        :param new_attname:
        :return:
        """
        logger.info(
            "Renaming g.attribute {0} into {1}".format(old_attname, new_attname)
        )
        # Check if the zVariable already exists
        if old_attname not in self.gattrs:
            logger.error("{0} g.attribute does not exist!".format(old_attname))
            return False
        else:
            self.gattrs[new_attname] = self.gattrs[old_attname]
            for i, entry in enumerate(self.gattrs[new_attname]):
                self.gattrs[new_attname][i]["Attribute Name"] = new_attname
            del self.gattrs[old_attname]
            return True

    def rm_gattr(self, attname):
        """
        Remove a global attr. from the Skeleton object.

        :param skeleton:
        :param attname:
        :return:
        """
        logger.info("Removing g.attribute {0}".format(attname))
        if attname in self.gattrs:
            del self.gattrs[attname]
            self.header["ngattr"] -= 1

    def add_vattr(self, attname, entry, varname=None, add=False):
        """
        Add entries for a variable attribute into the Skeleton object.

        :param attname: name of the v. attribute to add
        :param entry: List containing the entry for the v.attribute
        :param varname: If provided, contains a list of zvariable for which the v.attribute must be added. If not provided v.attribute is added for all the variables.
        :add: If True, add vattr entry to zvariable(s)
        :return:
        """
        if attname not in self.vattrList:
            self.vattrList.append(attname)
            self.header["nvattr"] += 1

        # Add Attribute Name value if not provided
        if "Attribute Name" not in entry:
            entry["Attribute Name"] = attname

        # Check that all the expected fields are in the input entries
        if not self.is_valid_vattrs(entry, ignore=["Variable Name"]):
            raise InvalidEntry

        # If varname not provided, then add variable attribute for all variables
        if not varname:
            varname = self.vattrs.keys()
        else:
            for var in varname:
                if var not in self.vattrs:
                    if not add:
                        logger.error("{0} not found in vattrs!".format(var))
                        raise InvalidEntry
                    else:
                        self.vattrs[var] = dict()

        for zvar in varname:
            logger.info("Adding v.attribute {0} for {1}".format(attname, zvar))
            entry["Variable Name"] = zvar
            self.vattrs[zvar][attname] = entry

        return True

    def set_vattr(self, attname, entry, varname=None):
        """
        Set a given v.attribute

        :param attname: name of the v. attribute to set
        :param entry: List containing the entry for the v.attribute
        :param varname: If provided, contains a list of zvariable for which the v.attribute must be set. If not provided v.attribute is set for all the variables.
        :return:
        """
        logger.info("Updating v.attribute {0}".format(attname))
        # Check if the attribute already exists
        if attname not in self.vattrList:
            logger.error("{0} v.attribute does not exist!".format(attname))
            return False
        else:

            # Variable name can be also provided in the entry
            if "Variable Name" in entry:
                varname = [entry["Variable Name"]]

            if not varname:
                varname = self.vattrs.keys()
            else:
                for var in varname:
                    if var not in self.vattrs:
                        logger.error("{0} not found!".format(var))
                        raise InvalidEntry

            for var in varname:
                for key, val in entry.items():
                    if key not in SHEETS[VATTRS]:
                        logger.warning("Unknown field: {0}!".format(key))
                        continue
                    self.vattrs[var][attname][key] = val

    def rename_vattr(self, old_attname, new_attname, varname=None):
        """
        Rename a variable attr. from the Skeleton object.

        :param old_attname: current name of the v.attribute
        :param new_attname: new new of the v.attribute
        :param varname: If provided, rename only for the given zVariables in the varname list. Else rename all.
        :return:
        """
        logger.info(
            "Renaming v.attribute {0} into {1}".format(old_attname, new_attname)
        )
        # Check if the zVariable already exists
        if old_attname not in self.vattrList:
            logger.error("{0} v.attribute does not exist!".format(old_attname))
            return False
        else:
            self.vattrList[new_attname] = self.vattrList[old_attname]
            del self.vattrList[old_attname]

            if old_attname in self.vattrList:
                self.vattrList[new_attname] = self.vattrList[old_attname]
                del self.vattrList[old_attname]

        if not varname:
            varname = self.vattrs.keys()
        else:
            for var in varname:
                if var not in self.vattrs:
                    logger.error("{0} not found!".format(var))
                    raise InvalidEntry

        # Check for each zvar too
        for var in varname:
            if old_attname in self.vattrs[var]:
                self.vattrs[var][new_attname] = self.vattrs[var][old_attname]
                del self.vattrs[var][old_attname]

        return True

    def rm_vattr(self, attname, varname=None):
        """
        Remove a variable attr. from the Skeleton object.

        :param attname: Name of the v.attribute to remove
        :param varname: if provided, only remove the v.attribute for the given list of zVariable in the varname input. Else remove all.
        :return:
        """
        logger.info("Removing v.attribute {0}".format(attname))
        if attname in self.vattrList:
            del self.vattrList[attname]
            self.header["nvattr"] -= 1

        if not varname:
            varname = self.vattrs.keys()
        else:
            for var in varname:
                if var not in self.vattrs:
                    logger.error("{0} not found!".format(var))
                    raise InvalidEntry

        # also check for each zvar
        for var in varname:
            if attname in self.vattrs[var]:
                del self.vattrs[var][attname]

    def add_zvar(self, varname, entry, vattrs=None):
        """
        Add a zVariable into the Skeleton object.

        :param varname: name of the zVariable to add
        :param entry: dictionary with fields to add [variable name, data type, number elements,
                    dims, sizes, record variance, Dimension variances]. If dims=0, sizes and record variance must be None
        :param vattrs: dictionnary containing variable attribute(s) to load for this zvariable. V.attributes can be also provided in the entry using the "Variable attributes" dictionary keyword.
        :return:
        """
        logger.info("Adding zvariable {0}".format(varname))
        # Check if the zVariable already exists
        if varname in self.zvars:
            logger.warning("{0} variable already exists!".format(varname))
            return False
        else:
            # Check content
            if not self.is_valid_zvar(entry):
                raise InvalidEntry
            # Add the zvariable
            self.zvars[varname] = entry
            self.header["nzvar"] += 1

            # if entry contains the "Variable Attributes" keyword, then extract it to get vattrs
            if "Variable Attribute" in entry:
                vattrs = entry["Variable Attributes"]

            # if Variable attributes are provided for this zVariable, then add them.
            if vattrs:
                for vattname, vattentry in vattrs.items():
                    self.add_vattr(vattname, vattentry, varname=[varname], add=True)

        return True

    def set_zvar(self, varname, entry, vattrs=None):
        """
        Update zvariable entry into the Skeleton object.

        :param varname:
        :param entry:
        :param vattrs:
        :return:
        """
        logger.info("Updating zvariable {0}".format(varname))
        # Check if the zVariable already exists
        if varname not in self.zvars:
            logger.error("{0} variable does not exist!".format(varname))
            return False
        else:
            # update the zvariable

            for field in SHEETS[ZVARS]:
                if field in entry:
                    self.zvars[varname][field] = entry[field]
                else:
                    continue

            # if entry contains the "Variable Attributes" keyword, then extract it to get vattrs
            if "Variable Attribute" in entry:
                vattrs = entry["Variable Attributes"]

            if vattrs:
                if varname not in self.vattrs:
                    logger.warning(
                        "{0} variable has no variable attribute, add input!".format(
                            varname
                        )
                    )
                    self.vattrs[varname] = dict()

                for key, val in vattrs.items():
                    if key not in SHEETS[VATTRS]:
                        logger.warning("Unknown field: {0}!".format(key))
                        continue
                    self.vattrs[varname][key] = val

        return True

    def rename_zvar(self, old_varname, new_varname):
        """
        Rename a zVariable of the Skeleton object.

        :param old_varname:
        :param new_varname:
        :return:
        """
        logger.info("Renaming zvariable {0} into {1}".format(old_varname, new_varname))
        # Check if the zVariable already exists
        if old_varname not in self.zvars:
            logger.error("{0} variable does not exist!".format(old_varname))
            return False
        else:
            self.zvars[new_varname] = self.zvars[old_varname]
            self.zvars[new_varname]["Variable Name"] = new_varname
            del self.zvars[old_varname]

            # Also rename in for the associated v.attribute(s)
            if old_varname in self.vattrs:
                self.vattrs[new_varname] = self.vattrs[old_varname]
                del self.vattrs[old_varname]

            return True

    def rm_zvar(self, varname):
        """
        Remove a zVariable from the Skeleton object.

        :param varname: name of the zVariable to remove
        :return:
        """
        logger.info("Removing zvariable {0}".format(varname))
        if varname in self.zvars:
            del self.zvars[varname]

        # Also remove associated v.attribute(s)
        if varname in self.vattrs:
            del self.vattrs[varname]

        self.header["nzvar"] -= 1
        return True


# ________________ Global Functions __________


# _________________ Main ____________________________
if __name__ == "__main__":
    print(__file__)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

"""Global constants for the package."""

import os
from importlib import metadata


__all__ = ["ROOT_DIRECTORY", "MASER_LIB_DIR", "SUPPORT_DIR", "__version__"]

ROOT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))

MASER_LIB_DIR = os.path.dirname(__file__)
SUPPORT_DIR = os.path.join(MASER_LIB_DIR, "support")
__version__ = metadata.version(__package__.replace(".", "-"))



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Constants of the maser.utils.time module."""

# ________________ IMPORT _________________________
# (Include here the modules to import, e.g. import sys)
from numpy import datetime64, timedelta64

__all__ = ["MJD_EPOCH", "JD_TO_MJD", "J2000_EPOCH", "TT2000_EPOCH", "DELTA_NSEC_TAI_TT"]

# ________________ HEADER _________________________

# # Mandatory
# __version__ = ""
# __author__ = ""
# __date__ = ""

# # Optional
# __license__ = ""
# __credit__ = [""]
# __maintainer__ = ""
# __email__ = ""
# __project__ = ""
# __institute__ = ""
# __changes__ = ""


# ________________ Global Variables _____________
# (define here the global variables)

# Time conversion factors
SEC_TO_MICROSEC = 1000000
DAY_TO_SEC = 24 * 3600

# Modified Julian day (MDJ) epoch (0h Nov 17, 1858)
MJD_EPOCH = datetime64("1858-11-17T00:00")

# delta time in microsec betwen Julian day (JD) epoch (-4713, 1, 1 at 12h)
# and MDJ epoch
JD_TO_MJD = timedelta64(2400000 * DAY_TO_SEC * SEC_TO_MICROSEC, "us") + timedelta64(
    12 * 3600 * SEC_TO_MICROSEC, "us"
)

# J2000 epoch
J2000_EPOCH = datetime64("2000-01-01T12:00")

# TT2000 epoch (including leap second)
TT2000_EPOCH = datetime64("2000-01-01T12:01:04.184")

# delta time in nanosec betwen TAI and TT times
# (TT = TAI + 32.184s; TT = UTC + delta_LeapSec + 32.184s)
DELTA_NSEC_TAI_TT = timedelta64(32184000000, "ns")

# ________________ Class Definition __________
# (If required, define here classes)

# ________________ Global Functions __________
# (If required, define here gobal functions)

# _________________ Main ____________________________
# if (__name__ == "__main__"):
# print ""
# main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Maser4py - Compare 2 CDF files module"""

import logging
import os
import os.path
import sys
from pprint import pformat

import numpy as np
from spacepy.pycdf import CDF

logger = logging.getLogger(__name__)


# Checking file
def checking_file_exist(cdf_file):
    if not os.path.isfile(cdf_file):
        raise FileNotFoundError("%s : does not exist !" % cdf_file)
    if not os.access(cdf_file, os.R_OK):
        raise IOError("%s : is not readable !" % cdf_file)


#
def get_keys_and_attributes(cdf_filepath):
    """
    Get the list data keys (zVars) and global attributes (gAttrs)

    :param cdf_filepath:
    :return:
    """
    with CDF(cdf_filepath) as cdf_file:
        return list(cdf_file.keys()), cdf_file.attrs.copy()


def list_differences(a, b):
    """
    Find list differences

    :param a: first list
    :param b: second list
    :return: a tuple containing the differences
    """
    return [x for x in a if x not in b], [x for x in b if x not in a]


# Listing all items
def list_elements(liste):
    n = len(liste)
    i = 0
    while i < n:
        # logger.debug("     %s", liste[i])
        i += 1


# Deleting a dictionnary's key
def delete_key(dict, key_to_remove):
    if key_to_remove in dict:
        del dict[key_to_remove]
    else:
        logger.debug("Key to remove '%s' does not exist !", key_to_remove)
    return dict


# Getting a given vAttr's not matched keys
def get_v_att_keys_diff(field1, field2):
    """
    get the differences between v_att keys of field 1 and field 2

    :param field1:
    :param field2:
    :return:
    """

    # convert v_att (class 'spacepy.cdf.zAttrList') of cdf files to sorted
    # lists

    list_a1 = sorted([attr_key for attr_key in field1.attrs])

    list_a2 = sorted([attr_key for attr_key in field2.attrs])
    if list_a1 != list_a2:
        result = list_differences(list_a1, list_a2)
    else:
        result = []

    return result


# Getting a given vAttr's matched keys
def get_matched_vAttrKey(field1, field2):
    set_a1 = set([attr_key for attr_key in field1.attrs])

    set_a2 = set([attr_key for attr_key in field2.attrs])

    return set_a1 & set_a2


def compare_global_attributes(global_att1, global_att2, list_ignore_gatt):
    """

    Compare global attributes

    :return: the global attribute differences
    """

    # initialize the Global Attributes dictionary
    gAttrs = {}

    list_global_att1 = sorted(list(global_att1.keys()))
    list_global_att2 = sorted(list(global_att2.keys()))

    if list_ignore_gatt != []:
        list_global_att1 = [n for n in list_global_att1 if n not in list_ignore_gatt]
        list_global_att2 = [n for n in list_global_att2 if n not in list_ignore_gatt]
    not_match_attribute = list_differences(list_global_att1, list_global_att2)

    l1 = len(not_match_attribute[0])
    l2 = len(not_match_attribute[1])

    if list_ignore_gatt != []:
        logger.debug(
            "%s Global Attributes to be ignored for comparison : %s",
            len(list_ignore_gatt),
            list_ignore_gatt,
        )
    if l1 != 0 or l2 != 0:
        logger.debug("Global attributes: different")
        logger.debug("File 1 : %s%s", str(len(global_att1)), " global attributes")
        logger.debug("File1's Global Attributes List : %s", list_global_att1)
        logger.debug("File 2 : %s%s", str(len(global_att2)), " global attributes")
        logger.debug("File2's Global Attributes List : %s", list_global_att2)
        logger.debug("Not matched global attributes")
        logger.debug(
            "   File1 : %s - %s", len(not_match_attribute[0]), not_match_attribute[0]
        )
        logger.debug(
            "   File2 : %s - %s", len(not_match_attribute[1]), not_match_attribute[1]
        )

        gAttrs["NotMatched"] = not_match_attribute
        # Remove not matched keys from the 2 dictionaries
        not_match1 = not_match_attribute[0]
        not_match2 = not_match_attribute[1]

        for key_to_remove in not_match1:
            global_att1 = delete_key(global_att1, key_to_remove)

        for key_to_remove in not_match2:
            global_att2 = delete_key(global_att2, key_to_remove)

    # Compare Global Attributes's value
    nl1 = len(global_att1)
    nl2 = len(global_att2)
    if nl1 == nl2:  # The 2 dictionaries have the same keys
        # Compare 2 dictionaries
        checking = global_att1 == global_att2
        if not checking:
            # Not equal !!
            logger.debug("Global attributes value: different")
            common_att = sorted(list(global_att1.keys()))

            DiffValueAttr = {}
            for com_att in common_att:
                if com_att not in list_ignore_gatt:
                    dd1 = global_att1.get(com_att)
                    dd2 = global_att2.get(com_att)
                    if dd1 != dd2:
                        val1 = dd1[0]
                        val2 = dd2[0]
                        logger.debug("** %s", com_att)
                        logger.debug("   File1 : %s", val1)
                        logger.debug("   File2 : %s", val2)

                        DiffValueAttr[com_att] = [val1, val2]
            if DiffValueAttr:
                gAttrs["Value"] = DiffValueAttr
            logger.debug("gAttrs:  %s", pformat(DiffValueAttr, width=1000))

    return gAttrs


def precision_dict_from_list(precision_list):
    precision_dict = {}
    for key_value in precision_list:
        key, value = key_value.split(":")
        precision_dict[key] = float(value)
    return precision_dict


def compare_z_var(
    field1,
    field2,
    key,
    dict_numerical_precision={},
    shape_diff_dict={},
    value_diff_dict={},
):
    # check if fields have the same shape
    if field1.shape:
        data1 = field1[...]
    else:
        # Make sure to have numpy ndarray if data1 is a scalar
        data1 = np.atleast_1d(field1[...])
    if field2.shape:
        data2 = field2[...]
    else:
        # Make sure to have numpy ndarray if data2 is a scalar
        data2 = np.atleast_1d(field2[...])

    res = False

    if data1.shape != data2.shape:
        logger.debug(
            "%s - array shape is different: %s | %s", key, field1.shape, field2.shape
        )
        shape_diff_dict[key] = [data1.shape, data2.shape]

    else:
        differences_mask = data1 != data2
        fields_are_different = np.any(differences_mask)

        # zVariable's key : check for different values
        if fields_are_different:
            if key in dict_numerical_precision.keys():
                tab_diff = np.isclose(data1, data2, atol=dict_numerical_precision[key])
                res = np.all(tab_diff)
                if not res:
                    logger.debug(
                        "Different values for zVariable '%s' : %s | %s",
                        key,
                        field1,
                        field2,
                    )
                    value_diff_dict[key] = [
                        data1[differences_mask],
                        data2[differences_mask],
                    ]

            else:
                logger.debug(
                    "Different values for zVariable '%s' : %s | %s", key, field1, field2
                )
                value_diff_dict[key] = [
                    data1[differences_mask],
                    data2[differences_mask],
                ]

    return shape_diff_dict, value_diff_dict


def compare_v_att(
    field1, field2, key, key_diff_dict={}, value_diff_dict={}, list_ignore_vatt=[]
):
    v_att_keys_diff = list_differences(field1.attrs, field2.attrs)

    if list_ignore_vatt != []:
        logger.debug(
            "%s Variable Attributes to be ignored for comparison : %s",
            len(list_ignore_vatt),
            list_ignore_vatt,
        )
        for item1 in list_ignore_vatt:
            if item1 in v_att_keys_diff[0]:
                (v_att_keys_diff[0]).remove(item1)
        for item2 in list_ignore_vatt:
            if item2 in v_att_keys_diff[1]:
                (v_att_keys_diff[1]).remove(item2)

    if len(v_att_keys_diff[0]) != 0 or len(v_att_keys_diff[1]) != 0:
        logger.debug(
            "Different Variable Attribute's keys of the zVariable '%s' : %s",
            key,
            v_att_keys_diff,
        )
        key_diff_dict[key] = v_att_keys_diff

    common_v_att_keys = get_matched_vAttrKey(field1, field2)
    logger.debug("Identical Variable Attribute's key : %s", common_v_att_keys)

    vAttrsList1 = field1.attrs
    vAttrsList2 = field2.attrs

    DiffValue_vAttr = {}

    for check_item in common_v_att_keys:
        logger.debug(
            "%s : %s  |  %s",
            check_item,
            vAttrsList1[check_item],
            vAttrsList2[check_item],
        )

        if vAttrsList1[check_item] == vAttrsList2[check_item]:
            logger.debug("%s : equal", check_item)
        else:
            logger.debug("%s : not equal", check_item)
            DiffValue_vAttr[check_item] = [
                vAttrsList1[check_item],
                vAttrsList2[check_item],
            ]
            value_diff_dict[key] = DiffValue_vAttr
            logger.debug("vAttrs['Value'] : %s", value_diff_dict)

    return key_diff_dict, value_diff_dict


def compare_data(
    cdf1,
    cdf2,
    cdf_keys1,
    cdf_keys2,
    list_ignore_zvar=[],
    list_ignore_vatt=[],
    list_numerical_precision=[],
):
    zVars = {}  # store data differences
    vAttrs = {}  # store attribute differences

    # Ignored zVariables for comparison
    if list_ignore_zvar != []:
        logger.debug(
            "%s zVariables to be ignored for comparison : %s",
            len(list_ignore_zvar),
            list_ignore_zvar,
        )

    if len(cdf_keys1) != len(cdf_keys2):
        logger.debug("Zvariables: different")

    # ***** Not matched keys *****
    list_diff1, list_diff2 = list_differences(cdf_keys1, cdf_keys2)

    if cdf_keys1 == cdf_keys2:
        logger.debug("Zvariables keys: identical")
        same_keys = cdf_keys1
        ordered_common_keys = sorted(list(same_keys))
    else:
        if (list_diff1 != []) or (list_diff2 != []):

            zVars["Keys"] = [list_diff1, list_diff2]

            logger.debug("NOT MATCHED zVARIABLES :")
            for idx, diff_list in enumerate(zVars["Keys"]):
                logger.debug("   File %d : %d - %s", idx + 1, len(diff_list), diff_list)
        # ***** Matched keys *****
        same_keys = set(cdf_keys1) & set(cdf_keys2)

        # ***** Alphabetical order *****
        ordered_common_keys = sorted(list(same_keys))

    logger.debug("MATCHED zVARIABLES : %d", len(ordered_common_keys))
    logger.debug("    %s", ordered_common_keys)

    # prepare the dicts to store z_var and v_att diff
    v_att_key_diff_dict = {}
    v_att_value_diff_dict = {}
    z_var_shape_diff_dict = {}
    z_var_value_diff_dict = {}

    for key in ordered_common_keys:
        if key in list_ignore_zvar:
            continue

        # Raw values comparison : It's really necessary for time values like "Epoch"
        #   cdf1.raw_var(key) => 549441617029459008
        #   cdf1[key] => 2017-05-30 18:39:07.845459
        field1 = cdf1.raw_var(key)
        field2 = cdf2.raw_var(key)

        dict_numerical_precision = {}
        if len(list_numerical_precision) != 0:
            dict_numerical_precision = precision_dict_from_list(
                list_numerical_precision
            )

        compare_z_var(
            field1,
            field2,
            key,
            dict_numerical_precision=dict_numerical_precision,
            shape_diff_dict=z_var_shape_diff_dict,
            value_diff_dict=z_var_value_diff_dict,
        )

        compare_v_att(
            field1,
            field2,
            key,
            list_ignore_vatt=list_ignore_vatt,
            key_diff_dict=v_att_key_diff_dict,
            value_diff_dict=v_att_value_diff_dict,
        )

    if v_att_value_diff_dict:
        vAttrs["Value"] = v_att_value_diff_dict
        logger.debug("vAttrs['Value'] : %s", vAttrs["Value"])

    if z_var_shape_diff_dict:
        zVars["Shape"] = z_var_shape_diff_dict
    if z_var_value_diff_dict:
        zVars["Value"] = z_var_value_diff_dict

    if v_att_key_diff_dict:
        vAttrs["Keys"] = v_att_key_diff_dict
    if v_att_value_diff_dict:
        vAttrs["Value"] = v_att_value_diff_dict

    return zVars, vAttrs


# Comparing 2 CDF files data
def cdf_compare(
    cdf_file1,
    cdf_file2,
    list_ignore_gatt=[],
    list_ignore_zvar=[],
    list_ignore_vatt=[],
    list_numerical_precision=[],
):
    logger.debug(" CDF file 1 : %s", cdf_file1)
    logger.debug(" CDF file 2 : %s", cdf_file2)
    checking_file_exist(cdf_file1)
    checking_file_exist(cdf_file2)

    cdf_keys1, global_att1 = get_keys_and_attributes(cdf_file1)
    cdf_keys2, global_att2 = get_keys_and_attributes(cdf_file2)

    cdf1 = CDF(cdf_file1)
    cdf2 = CDF(cdf_file2)

    dict_result = {}

    gAttrs = compare_global_attributes(global_att1, global_att2, list_ignore_gatt)

    if gAttrs:
        dict_result["gAttrs"] = gAttrs

    zVars, vAttrs = compare_data(
        cdf1,
        cdf2,
        cdf_keys1,
        cdf_keys2,
        list_ignore_zvar=list_ignore_zvar,
        list_ignore_vatt=list_ignore_vatt,
        list_numerical_precision=list_numerical_precision,
    )

    if zVars:
        dict_result["zVars"] = zVars

    if vAttrs:
        dict_result["vAttrs"] = vAttrs

    cdf1.close()
    cdf2.close()

    for key, value in dict_result.items():
        logger.debug("*°*°* %s *°*°*", key)
        for key1, value1 in dict_result[key].items():
            logger.debug("     *°* %s : %s", key1, value1)

    # Case of we need to force to ignore some zVariables
    # if forced_ignored_zvar != []:
    #     logger.debug("Forced ignored zVariables (Particular case) : %s", forced_ignored_zvar)

    logger.debug("Return value: %s", pformat(dict_result, width=1000))
    return dict_result


def main(cdf_file1, cdf_file2):
    if len(sys.argv) >= 3:

        list_argv = sys.argv
        list_ignore_gatt = []
        list_ignore_zvar = []
        list_ignore_vatt = []
        list_numerical_precision = []

        if "--ignore_gatt" in (list_argv):
            ind_ignore_gatt = (list_argv).index("--ignore_gatt")
            list_ignore_gatt = []
            for item in list_argv[ind_ignore_gatt + 1 :]:
                if not item.startswith("--ignore_") and not item.startswith(
                    "--precision"
                ):
                    list_ignore_gatt.append(item)
                else:
                    break
            logger.warning("Ignored global attributes list : %s", list_ignore_gatt)
        else:
            logger.debug("No global attributes to be ignored")

        if "--ignore_zvar" in (list_argv):
            ind_ignore_zvar = (list_argv).index("--ignore_zvar")
            list_ignore_zvar = []
            for item in list_argv[ind_ignore_zvar + 1 :]:
                if not item.startswith("--ignore_") and not item.startswith(
                    "--precision"
                ):
                    list_ignore_zvar.append(item)
                else:
                    break
            logger.warning("Ignored zVariables list : %s", list_ignore_zvar)
        else:
            logger.debug("No zVariables to be ignored")

        if "--ignore_vatt" in (list_argv):
            ind_ignore_vatt = (list_argv).index("--ignore_vatt")
            list_ignore_vatt = []
            for item in list_argv[ind_ignore_vatt + 1 :]:
                if not item.startswith("--ignore_") and not item.startswith(
                    "--precision"
                ):
                    list_ignore_vatt.append(item)
                else:
                    break
            logger.warning("Ignored variable attributes list : %s", list_ignore_vatt)
        else:
            logger.debug("No variable attributes to be ignored")

        if "--precision" in (list_argv):
            ind_precision_zvar = (list_argv).index("--precision")
            list_numerical_precision = []
            for item in list_argv[ind_precision_zvar + 1 :]:
                if not item.startswith("--ignore_") and not item.startswith(
                    "--precision"
                ):
                    list_numerical_precision.append(item)
                else:
                    break
            logger.warning("Numerical precision list : %s", list_numerical_precision)
        else:
            logger.debug("No zVariable precision set")

        if "--precision" in (list_argv):
            ind_precision_zvar = (list_argv).index("--precision")
            list_numerical_precision = []
            for item in list_argv[ind_precision_zvar + 1 :]:
                if not item.startswith("--ignore_") and not item.startswith(
                    "--precision"
                ):
                    list_numerical_precision.append(item)
                else:
                    break
            logger.warning("Numerical precision list : %s", list_numerical_precision)
        else:
            logger.debug("No zVariable precision set")

        result = cdf_compare(
            cdf_file1,
            cdf_file2,
            list_ignore_gatt=list_ignore_gatt,
            list_ignore_zvar=list_ignore_zvar,
            list_ignore_vatt=list_ignore_vatt,
            list_numerical_precision=list_numerical_precision,
        )
        logger.info("Final result : %s", pformat(result, width=1000))


# *°*°*°*°*°*°*°*°
#  Main program
# *°*°*°*°*°*°*°*°

if __name__ == "__main__":
    main(sys.argv[1], sys.argv[2])



# -*- coding: utf-8 -*-
import os
from maser.tools.settings import SUPPORT_DIR

JINJA_TEMPLATE_DIR = os.path.join(SUPPORT_DIR, "cdf")

SHEETS = {
    "header": [
        "CDF NAME",
        "DATA ENCODING",
        "MAJORITY",
        "FORMAT",
        "CDF_COMPRESSION",
        "CDF_CHECKSUM",
    ],
    "GLOBALattributes": ["Attribute Name", "Entry Number", "Data Type", "Value"],
    "zVariables": [
        "Variable Name",
        "Data Type",
        "Number Elements",
        "Dims",
        "Sizes",
        "Record Variance",
        "Dimension Variances",
        "VAR_COMPRESSION",
        "VAR_SPARESERECORDS",
        "VAR_PADVALUE",
    ],
    "VARIABLEattributes": ["Variable Name", "Attribute Name", "Data Type", "Value"],
    "NRV": ["Variable Name", "Index", "Value"],
}

HEADER = "header"
GATTRS = "GLOBALattributes"
VATTRS = "VARIABLEattributes"
ZVARS = "zVariables"
RVARS = "variables"
NRV = "NRV"
END = "end"

NEW_HEADER = "!VariablesG.AttributesV.AttributesRecordsDimsSizes"
NEW_ZVAR = "!VariableDataNumberRecordDimension"
NEW_VATTRS = "!AttributeData"
NO_NRV = "!RVvalueswerenotrequested."
NEW_NRV = "!NRVvaluesfollow..."



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Maser4py time module."""

# ________________ IMPORT _________________________
# (Include here the modules to import, e.g. import sys)
from pytz import timezone
import numpy
from numpy import datetime64, timedelta64
from datetime import datetime, timedelta
import logging

from ..toolbox import print_exception

from .leapsec import Lstable

from .const import MJD_EPOCH, JD_TO_MJD, TT2000_EPOCH, DELTA_NSEC_TAI_TT

__all__ = [
    "set_tzone",
    "local_to_utc",
    "jd_to_mjd",
    "jd_to_tt2000",
    "mjd_to_jd",
    "tai_to_tt",
    "tt_to_tai",
    "tt_to_utc",
    "tt_to_tt2000",
    "utc_to_tt2000",
    "utc_to_tt",
    "tt2000_to_tt",
    "tt2000_to_utc",
    "tt2000_to_jd",
    "get_leapsec",
]

# ________________ HEADER _________________________

# Mandatory
__version__ = "0.1.0"
__author__ = "X.Bonnin"
__date__ = "2017-03-31"

# Optional
__license__ = ""
__credit__ = [""]
__maintainer__ = ""
__email__ = ""
__project__ = "MASER"
__institute__ = "LESIA"
__changes__ = ""


# ________________ Global Variables _____________
# (define here the global variables)
logger = logging.getLogger(__name__)

# ________________ Class Definition __________
# (If required, define here classes)


class TimeException(Exception):
    pass


# ________________Global Functions __________
# (If required, define here classes)


def cast_timedelta(fr=None, to=None):
    """
    cast_timedelta decorator.

    Decorator to set the first input/output argument of a
    function as a datetime.timedelta or
    numpy.timedelta64 type object depending of
    the decorator input keywords.

    Inputs:
        fr -- keyword to indicate the type to be passed to
              the function.
              (Possible values = timedelta, timedelta64 or None).
              If 'fr=None', then use the input type.
        to -- keyword to indicate the type to be returned by
              the function.
              (Possible values = timedelta, timedelta64 or None).
              If 'fr=None', then use the input type.

    Example:

        from datetime import timedelta
        from numpy import timedelta64

        @cast_timedelta(to=timedelta, fr=timedelta64)
        def func(td_in):
            ...
            return td_out

        Calling 'td_out = func(td_in)' will force the input 'td_in'
        parameter to be passed to the function as a
        datetime.timedelta object and to
        return the td_out output as a numpy.timedelta64 object.
    """

    def decorated(func):
        """Decorated method."""

        def wrapper(*args, **kwargs):
            args = list(args)
            """Decorator wrapper."""
            type_in = type(args[0])
            try:
                if type_in is not timedelta and type_in is not timedelta64:
                    # Only timedelta and timedelta64 objects are allowed as
                    # input.
                    raise TypeError(
                        "Input argument format is not valid [{0}]!".format(type_in)
                    )

                if type_in is timedelta64 and fr is timedelta:
                    td_in = td64_to_td(args[0])
                elif type_in is timedelta and fr is timedelta64:
                    td_in = td_to_td64(args[0])
                elif type_in is timedelta64 and fr is timedelta64:
                    td_in = args[0]
                elif type_in is timedelta and fr is timedelta:
                    td_in = args[0]
                elif fr is None:
                    td_in = args[0]
                else:
                    raise TypeError("Input argument is not valid!")
            except Exception:
                print_exception()
            else:
                # Execute the function with the right input time type
                args[0] = td_in
                td_out = func(*args, **kwargs)

            # Depending of the decorator input and function I/O
            # return the output timedelta with the right time type
            type_out = type(td_out)
            if type_out is timedelta64 and to is timedelta:
                return td64_to_td(td_out)
            elif type_out is timedelta and to is timedelta64:
                return td_to_td64(td_out)
            elif type_out is timedelta and to is timedelta:
                return td_out
            elif type_out is timedelta64 and to is timedelta64:
                return td_out
            # If output type is not defined, then return as an input type
            elif type_out is timedelta and to is None and type_in is timedelta:
                return td_out
            elif type_out is timedelta and to is None and type_in is timedelta64:
                return td_to_td64(td_out)
            elif type_out is timedelta64 and to is None and type_in is timedelta:
                return td64_to_td(td_out)
            elif type_out is timedelta64 and to is None and type_in is timedelta64:
                return td_out
            else:
                return td_out

        return wrapper

    return decorated


def cast_datetime(fr=None, to=None):
    """
    cast_datetime decorator.

    Decorator to set the first input/output argument of a
    function as a datetime.datetime or
    numpy.datetime64 type object depending of
    the decorator input keywords.

    Inputs:
        fr -- keyword to indicate the type to be passed to
              the function.
              (Possible values = datetime, datetime64 or None).
              If 'fr=None', then use the input type.
        to -- keyword to indicate the type to be returned by
              the function.
              (Possible values = datetime, datetime64 or None).
              If 'fr=None', then use the input type.

    Example:

        from datetime import datetime
        from numpy import datetime64

        @cast_datetime(to=datetime, fr=datetime64)
        def func(dt_in):
            ...
            return dt_out

        Calling 'dt_out = func(dt_in)' will force the 'dt_in' input parameter
        to be passed to the function as a datetime.datetime object and to
        return the dt_out output as a numpy.datetime64 object.
    """

    def decorated(func):
        """Decorated method."""

        def wrapper(*args, **kwargs):
            args = list(args)
            """Decorator wrapper."""
            args = list(args)
            type_in = type(args[0])
            try:
                if type_in is not datetime and type_in is not datetime64:
                    # Only datetime and datetime64 objects are allowed as
                    # input.
                    raise TypeError(
                        "Input argument format is not valid [{0}]!".format(type_in)
                    )

                if type_in is datetime64 and fr is datetime:
                    dt_in = dt64_to_dt(args[0])
                elif type_in is datetime and fr is datetime64:
                    dt_in = dt_to_dt64(args[0])
                elif type_in is datetime64 and fr is datetime64:
                    dt_in = args[0]
                elif type_in is datetime and fr is datetime:
                    dt_in = args[0]
                elif fr is None:
                    dt_in = args[0]
                else:
                    raise TypeError("Input argument is not valid!")
            except Exception:
                print_exception()
            else:
                # Execute the function with the right input time type
                args[0] = dt_in
                dt_out = func(*args, **kwargs)

            # Depending of the decorator input and function I/O
            # return the output datetime with the right time type
            type_out = type(dt_out)
            if type_out is datetime64 and to is datetime:
                return dt64_to_dt(dt_out)
            elif type_out is datetime and to is datetime64:
                return dt_to_dt64(dt_out)
            elif type_out is datetime and to is datetime:
                return dt_out
            elif type_out is datetime64 and to is datetime64:
                return dt_out
            # If output type is not defined, then return as an input type
            elif type_out is datetime and to is None and type_in is datetime:
                return dt_out
            elif type_out is datetime and to is None and type_in is datetime64:
                return dt_to_dt64(dt_out)
            elif type_out is datetime64 and to is None and type_in is datetime:
                return dt64_to_dt(dt_out)
            elif type_out is datetime64 and to is None and type_in is datetime64:
                return dt_out
            else:
                return dt_out

        return wrapper

    return decorated


@cast_datetime(fr=datetime)
def get_leapsec(date, leapsec_file=None, to_timedelta64=False):
    """Return leap seconds in timedelta format for a given datetime.

    Leapsec are returned in the timedelta64 format if
    to_timedelta64 keyword is set to True.
    """
    leapsec = Lstable(file=leapsec_file).get_leapsec(date)

    # Convert seconds in float to timedelta object
    leapsec = timedelta(microseconds=leapsec * 1000000)

    if to_timedelta64:
        leapsec = numpy.timedelta64(leapsec)

    # get leapsec.
    return leapsec


@cast_datetime(fr=datetime)
def set_tzone(date, tzone):
    """Change the current time zone."""
    return date.replace(timezone(tzone))


def td64_to_td(td64):
    """
    td64_to_td.

    Convert a numpy.timedelta64 object into a
    datetime.timedelta object.
    """
    return td64.astype(timedelta)


def td_to_td64(td):
    """
    td_to_td64.

    Convert a numpy.timedelta object into a
    numpy.timedelta64 object.
    """
    return numpy.timedelta64(td)


def dt64_to_dt(dt64):
    """
    dt64_to_dt.

    Convert a numpy.datetime64 object into a
    datetime.datetime object.

    Note that the time resolution of a datetime.datetime
    object is microsecond.
    """
    return dt64.astype("M8[us]").astype("O")


def dt_to_dt64(dt):
    """
    dt_to_dt64.

    Convert a numpy.datetime object into a
    numpy.datetime64 object.

    Note that the time resolution of a numpy.datetime
    object is attosecond.
    """
    return numpy.datetime64(dt)


@cast_datetime(fr=datetime)
def local_to_utc(local_time, tzone=None):
    """
    local_to_utc.

    From a given local time and timezone,
    returns the corresponding UTC time.

    Input parameter local_time must be a
    datetime.datetime object.

    See pytz module documentation for allowed values
    of the input optional parameter timezone

    UTC time is returned as a datetime.datetime object.
    """
    if tzone is not None:
        tz = timezone(tzone)
        ltime = tz.localize(local_time.replace(tzinfo=None))

    return ltime.astimezone(timezone("UTC"))


@cast_timedelta(fr=timedelta64)
def jd_to_mjd(jd):
    """Convert Julian Days (JD) to modified JD.

    Be awared that the best time resolution is
    microsec.
    """
    return numpy.timedelta64(jd, "us") - JD_TO_MJD


@cast_timedelta(fr=timedelta64)
def mjd_to_jd(mjd):
    """Convert Modified Julian Days (MJD) to JD.

    Be awared that the best time resolution is
    microsec.
    """
    return numpy.timedelta64(mjd, "us") + JD_TO_MJD


@cast_datetime(fr=datetime64)
def tai_to_tt(tai):
    """Convert TAI time to TT time."""
    return tai + DELTA_NSEC_TAI_TT


@cast_datetime(fr=datetime64)
def tt_to_tai(tt):
    """Convert TT time to TAI time."""
    return tt - DELTA_NSEC_TAI_TT


# Works with np.datetime64 object I/O only
@cast_datetime(fr=datetime64, to=datetime64)
def utc_to_tt(utc):
    """
    utc_to_tt.

    Convert UTC time to Terrestrial Time (TT).
    TT = TAI + DELTA_NSEC_TAI_TT;
    TT = UTC + delta_LeapSec + DELTA_NSEC_TAI_TT
    where DELTA_NSEC_TAI_TT = 32.184s

    TT time is returned as a numpy.datetime64 object.
    """
    return utc + get_leapsec(utc, to_timedelta64=True) + DELTA_NSEC_TAI_TT


@cast_datetime(fr=datetime64)
def tt_to_utc(tt):
    """
    utc_to_tt.

    Convert UTC time to Terrestrial Time (TT).
    TT = TAI + DELTA_NSEC_TAI_TT;
    TT = UTC + delta_LeapSec + DELTA_NSEC_TAI_TT
    where DELTA_NSEC_TAI_TT = 32.184s

    """
    return tt - get_leapsec(tt, to_timedelta64=True) - DELTA_NSEC_TAI_TT


@cast_datetime(fr=datetime64)
def utc_to_jd(utc, to_mjd=True):
    """
    utc_to_jd.

    Convert UTC time into Julian days.

    Returns MJD instead of JD if to_mjd keyword is set.

    Returned value is always a numpy.timedelta64 object.
    """
    mjd = utc - MJD_EPOCH

    if not to_mjd:
        return mjd_to_jd(mjd)
    else:
        return mjd


@cast_timedelta(fr=timedelta64)
def jd_to_utc(jd, from_mjd=True):
    """
    jd_to_utc.

    Convert Julian Days into UTC time.

    Use MJD instead of JD as input if from_mjd keyword is set.

    Returned value is always a numpy.datetime64 object.
    """
    if not from_mjd:
        return jd_to_mjd(jd) + MJD_EPOCH
    else:
        return jd + MJD_EPOCH


@cast_datetime(fr=datetime64)
def utc_to_tt2000(utc):
    """
    utc_to_tt2000 method.

    Convert UTC time into TT2000 time.

    Returned value is a numpy.timedelta64 object.
    """
    tt = utc_to_tt(utc)
    return tt_to_tt2000(tt)


# Works with np.datetime64 object I/O only
@cast_timedelta(fr=timedelta64)
def tt2000_to_utc(tt2000):
    """
    tt2000_to_utc method.

    Convert TT2000 time into UTC.

    Returned value is a numpy.datetime64 object.
    """
    tt = tt2000_to_tt(tt2000)
    return tt_to_utc(tt)


@cast_timedelta(fr=timedelta64)
def tt2000_to_tt(tt2000):
    """
    tt2000_to_tt.

    Convert TT2000 time into TT.

    Returns TT in timedelta64 format.
    """
    return tt2000 + TT2000_EPOCH


@cast_datetime(fr=datetime64)
def tt_to_tt2000(tt):
    """
    tt_to_tt2000.

    Convert TT time into TT2000.

    Returns TT2000 in timedelta64 format.
    """
    return tt - TT2000_EPOCH


@cast_timedelta(fr=timedelta64)
def tt2000_to_jd(tt2000, to_mjd=False):
    """
    tt2000_to_jd.

    Convert TT2000 time into Julian days.

    Returns MJD instead of JD if to_mjd keyword is set.

    Returned value is a numpy.timedelta64 object
    with microsec. time resolution.

    """
    # Convert TT2000 in MJD
    utc = tt2000_to_utc(tt2000)
    return utc_to_jd(utc, to_mjd=to_mjd)


@cast_timedelta(fr=timedelta64)
def jd_to_tt2000(jd, from_mjd=False):
    """
    jd_to_tt2000.

    Convert Julian Days (JD) into TT2000.

    Use MJD instead of JD if from_mjd keyword is set.

    Returned value is always a numpy.timedelta64 object.

    The best time resolution of input JD must be microsec.

    """
    # Convert to MJD, if from_mjd keyword is not set
    utc = jd_to_utc(jd, from_mjd=from_mjd)
    return utc_to_tt2000(utc)


# _________________ Main ____________________________
# if (__name__ == "__main__"):
# print ""
# main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Module providing additional tools for the CDF format handling
"""

# ________________ IMPORT _________________________
# (Include here the modules to import, e.g. import sys)
import logging

from spacepy.pycdf import const, zAttr
import numpy

__all__ = ["get_cdftype", "get_vattrs", "get_cdftypename", "get_numpttype"]

# ________________ HEADER _________________________

# ________________ Global Variables _____________
# (define here the global variables)
logger = logging.getLogger(__name__)

# ________________ Global Functions __________
# (If required, define here gobal functions)


def get_cdftype(dtype):
    """
    Return the id of a cdf data type,
    prividing the CDF data type.
    """
    return const.__dict__[dtype].value


def get_cdftypename(dtype=None):
    """
    Return the CDF type name of a given cdf cdf type
    Return the full CDF data type dictionnary if no input
    argument is provided.
    """
    cdftypenames = {
        const.CDF_BYTE.value: "CDF_BYTE",
        const.CDF_CHAR.value: "CDF_CHAR",
        const.CDF_INT1.value: "CDF_INT1",
        const.CDF_UCHAR.value: "CDF_UCHAR",
        const.CDF_UINT1.value: "CDF_UINT1",
        const.CDF_INT2.value: "CDF_INT2",
        const.CDF_UINT2.value: "CDF_UINT2",
        const.CDF_INT4.value: "CDF_INT4",
        const.CDF_UINT4.value: "CDF_UINT4",
        const.CDF_INT8.value: "CDF_INT8",
        const.CDF_FLOAT.value: "CDF_FLOAT",
        const.CDF_REAL4.value: "CDF_REAL4",
        const.CDF_DOUBLE.value: "CDF_DOUBLE",
        const.CDF_REAL8.value: "CDF_REAL8",
        const.CDF_EPOCH.value: "CDF_EPOCH",
        const.CDF_EPOCH16.value: "CDF_EPOCH16",
        const.CDF_TIME_TT2000.value: "CDF_TIME_TT2000",
    }

    if dtype is None:
        return cdftypenames
    else:
        return cdftypenames[dtype]


def get_numpttype(dtype=None):
    """
    Return the numpy data type from a given cdf data type.
    Return the full numpy data type dictionnary if no input
    argument is provided.
    """

    numpytypedict = {
        const.CDF_BYTE.value: numpy.int8,
        const.CDF_CHAR.value: numpy.int8,
        const.CDF_INT1.value: numpy.int8,
        const.CDF_UCHAR.value: numpy.uint8,
        const.CDF_UINT1.value: numpy.uint8,
        const.CDF_INT2.value: numpy.int16,
        const.CDF_UINT2.value: numpy.uint16,
        const.CDF_INT4.value: numpy.int32,
        const.CDF_UINT4.value: numpy.uint32,
        const.CDF_INT8.value: numpy.int64,
        const.CDF_FLOAT.value: numpy.float32,
        const.CDF_REAL4.value: numpy.float32,
        const.CDF_DOUBLE.value: numpy.float64,
        const.CDF_REAL8.value: numpy.float64,
        const.CDF_EPOCH.value: numpy.float64,
        const.CDF_EPOCH16.value: numpy.dtype((numpy.float64, 2)),
        const.CDF_TIME_TT2000.value: numpy.int64,
    }

    if dtype is None:
        return numpytypedict
    else:
        return numpytypedict[dtype]


def get_vattrs(cdf):

    """
    Retrieve the list of zVariables attributes
    from a given cdf object
    """

    vattrs = {}

    if len(cdf) == 0:
        logger.warning("No Zvariable found!")
        return vattrs

    for zvar in cdf:
        zattrs = cdf[zvar].attrs
        for vattr in zattrs:
            zattr = zAttr(cdf, vattr)
            vattrs[vattr] = zattr

    return vattrs



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from maser.tools.cdf.cdfcompare.subparser import add_cdfcompare_subparser  # noqa: F401
from maser.tools.cdf.cdfcompare.cdf_compare import cdf_compare  # noqa: F401



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .leapsec import Lstable  # noqa: F401
from .time import *  # noqa: F401, F403
from .const import *  # noqa: F401, F403
from .subparser import add_leapsec_subparser  # noqa: F401



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""cdfcompare module subparser."""

# ________________ IMPORT _________________________
# (Include here the modules to import, e.g. import sys)

# ________________ HEADER _________________________


# ________________ Global Variables _____________
# (define here the global variables)

# ________________ Class Definition __________
# (If required, define here classes)

# ________________ Global Functions __________
# (If required, define here global functions)
def add_cdfcompare_subparser(subparser):
    """cdf.cdfcompare script program."""
    cdfcompare_parser = subparser.add_parser("cdf_compare", help="Compare 2 CDF files.")
    cdfcompare_parser.add_argument("cdf_filepath1", help="Input CDF file path 1")
    cdfcompare_parser.add_argument("cdf_filepath2", help="Input CDF file path 2")
    cdfcompare_parser.add_argument(
        "--ignore_gatt", nargs="+", default=[], help="Global attributes to ignore"
    )
    cdfcompare_parser.add_argument(
        "--ignore_zvar", nargs="+", default=[], help="Global attributes to ignore"
    )
    cdfcompare_parser.add_argument(
        "--ignore_vatt", nargs="+", default=[], help="Global attributes to ignore"
    )
    cdfcompare_parser.add_argument(
        "--precision", nargs="+", default=[], help="zVariable precision settings"
    )

    # _________________ Main ____________________________


# if __name__ == "__main__":
#     print(__file__)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Python module containing the skeletoncdf subparser."""

# ________________ IMPORT _________________________
# (Include here the modules to import, e.g. import sys)
import os

__all__ = ["add_skeletoncdf_subparser", "add_skeletontable_subparser"]

# ________________ HEADER _________________________


# ________________ Global Variables _____________
# (define here the global variables)

# ________________ Class Definition __________
# (If required, define here classes)

# ________________ Global Functions __________
# (If required, define here global functions)
def add_skeletoncdf_subparser(subparser):
    """
    Subparser for cdf.serializer.skeletoncdf.

    :param subparser:
    :return:
    """
    sktcdf_parser = subparser.add_parser(
        "skeletoncdf", help="Convert CDF skeleton table into binary CDF"
    )
    sktcdf_parser.add_argument(
        "skeletons", nargs="+", help="Input CDF skeleton file(s)"
    )
    sktcdf_parser.add_argument(
        "-o",
        "--output_dir",
        nargs=1,
        default=[os.getcwd()],
        help="Path of the output directory",
    )
    sktcdf_parser.add_argument(
        "-e",
        "--executable",
        nargs=1,
        default=[None],
        help="Path of the skeletoncdf binary executable of the NASA CDF Toolkit.",
    )
    sktcdf_parser.add_argument(
        "-O", "--overwrite", action="store_true", help="Overwrite existing output files"
    )
    sktcdf_parser.add_argument(
        "-F",
        "--force",
        action="store_true",
        help="Force conversions even if an exception has raised",
    )
    sktcdf_parser.add_argument(
        "--no-auto-pad",
        action="store_true",
        help="Deactivate automated assignment of the !VAR_PADVALUE",
    )
    sktcdf_parser.add_argument(
        "--no-cdf", action="store_true", help='Do no generate output CDF "master" file'
    )


def add_skeletontable_subparser(subparser):
    """
    Subparser for cdf.serializer.skeletontable.

    :param subparser:
    :return:
    """
    cdfskt_parser = subparser.add_parser(
        "skeletontable", help="Convert CDF file into skeleton table"
    )
    cdfskt_parser.add_argument("cdf", nargs="+", help="Input CDF file(s)")
    cdfskt_parser.add_argument(
        "-x",
        "--to-xlsx",
        action="store_true",
        help="Input CDF also saved as Excel 2007 format file(s)",
    )
    cdfskt_parser.add_argument(
        "-o",
        "--output_dir",
        nargs=1,
        default=[os.getcwd()],
        help="Path of the output directory",
    )
    cdfskt_parser.add_argument(
        "-e",
        "--executable",
        nargs=1,
        default=[None],
        help="Path of the skeletontable binary executable of the NASA CDF Toolkit.",
    )
    cdfskt_parser.add_argument(
        "-O", "--overwrite", action="store_true", help="Overwrite existing output files"
    )
    cdfskt_parser.add_argument(
        "-F",
        "--force",
        action="store_true",
        help="Force conversions even if an exception has raised",
    )

    # _________________ Main ____________________________


# if __name__ == "__main__":
#     print(__file__)
